header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.generated;
	import org.antlr_yggdrasil.tool.Rule;
	import org.antlr_yggdrasil.tool.Grammar;
	import org.antlr_yggdrasil.tool.GrammarAST;
	import org.antlr_yggdrasil.tool.ErrorManager;
	import org.antlr_yggdrasil.codegen.CodeGenerator;
	import org.antlr_yggdrasil.analysis.Label;
	import org.antlr_yggdrasil.analysis.DFA;
	import org.antlr_yggdrasil.misc.IntSet;
	import org.antlr_yggdrasil.runtime.TypeInfo;
	import org.antlr_yggdrasil.runtime.ArrayInfo;
	import java.util.*;
	import org.antlr.stringtemplate.*;
    import antlr.TokenWithIndex;
    import antlr.CommonToken;
}

/** Walk a grammar and generate code by gradually building up
 *  a bigger and bigger StringTemplate.
 *
 *  Terence Parr
 *  University of San Francisco
 *  June 15, 2004
 */
class YggdrasilCodeGen extends TreeParser;

options {
    // warning! ANTLR cannot see another directory to get vocabs, so I had
    // to copy the ANTLRTokenTypes.txt file into this dir from ../tools!
    // Yuck!  If you modify ../tools/antlr.g, make sure to copy the vocab here.
	importVocab = Yggdrasil;
    codeGenBitsetTestThreshold=999;
    ASTLabelType=GrammarAST;
}

{
	protected static final int RULE_BLOCK_NESTING_LEVEL = 0;
	protected static final int OUTER_REWRITE_NESTING_LEVEL = 0;

    protected String currentRuleName = null;
    protected int blockNestingLevel = 0;
	protected int outerAltNum = 0;
    protected StringTemplate currentBlockST = null;

    public void reportError(RecognitionException ex) {
		Token token = null;
		if ( ex instanceof MismatchedTokenException ) {
			token = ((MismatchedTokenException)ex).token;
		}
		else if ( ex instanceof NoViableAltException ) {
			token = ((NoViableAltException)ex).token;
		}
        ErrorManager.syntaxError(
            ErrorManager.MSG_SYNTAX_ERROR,
            token,
            "codegen: "+ex.toString(),
            ex);
    }

    public void reportError(String s) {
        System.out.println("codegen: error: " + s);
    }

    protected CodeGenerator generator;
    protected Grammar grammar;
    protected StringTemplateGroup templates;
    protected StringTemplateGroup userTemplates;
    protected boolean inSynPred = false;
    protected boolean addingAttributes = true;
    protected boolean killChars = false;
    protected boolean isTree = true;

    /** The overall lexer/parser template; simulate dynamically scoped
     *  attributes by making this an instance var of the walker.
     */
    protected StringTemplate recognizerST;

    protected StringTemplate outputFileST;
    protected StringTemplate headerFileST;
    protected StringTemplate factoryFileST;
    protected StringTemplate vpublic;
    protected StringTemplate variables;

    protected String outputOption = "";

	protected StringTemplate getWildcardST(GrammarAST elementAST, GrammarAST ast_suffix, String label) {
		if ( grammar.type==Grammar.LEXER ) {
			StringTemplate retval = templates.getInstanceOf("wildcardChar");
			if (killChars) {
				retval.setAttribute("skip", "true");
			}
			return retval;	
		}
		else
			return templates.getInstanceOf("wildcard");
	}


    protected void init(Grammar g) {
        this.grammar = g;
        this.generator = grammar.getCodeGenerator();
        this.templates = generator.getTemplates();
	vpublic = templates.getInstanceOf("public");
	variables = templates.getInstanceOf("variables");
	factoryFileST = templates.getInstanceOf("factory");
    }

	protected void setAccessorNames(StringTemplate a, String nm) {
		String firstChar = nm.substring(0,1).toUpperCase();
		String uname = firstChar + nm.substring(1);
		a.setAttribute("name", nm);
		a.setAttribute("uname", uname);
	}

	protected String getUname(String nm) {
		String firstChar = nm.substring(0,1).toUpperCase();
		String uname = firstChar + nm.substring(1);

		return uname;
	}

	protected String decodeName(String nm) {
		String firstChar = nm.substring(0,1).toLowerCase();
		String name = firstChar + nm.substring(1);

		return name;
	}

	public StringTemplate getFactoryTemplate() {
		return factoryFileST;
	}

	public void setFactoryTemplate(StringTemplate factory) {
		factoryFileST = factory;
	}

	public void setUserTemplates(StringTemplateGroup lib) {
		userTemplates = lib;
	}

	protected StringTemplate lastValueOf(StringTemplate t, String attr) {
		Object l = t.getAttribute(attr);
		StringTemplate retval = null;

		if (l instanceof StringTemplate.STAttributeList) {
			StringTemplate.STAttributeList ls = (StringTemplate.STAttributeList) l;
			retval = (StringTemplate) ls.get(ls.size() - 1);
		}
		else
			retval = (StringTemplate) l;

		return retval;
	}

	protected StringTemplate addAST(StringTemplate assigned, String dir) {
		StringTemplate tpl = lastValueOf(assigned, "index");
		String uname = (String) tpl.getAttribute("name");
		String typeStr = (String) tpl.getAttribute("type");
		TypeInfo type = grammar.getType(typeStr);
		return addAST(assigned.toString(), dir, type);
	}

	protected StringTemplate addAST(String fetch,
			String dir, TypeInfo type) {
		StringTemplate st = null;

		if (type.isa("Payload")) {
			st = templates.getInstanceOf("instantiatePayload");
		}
		else if (type.isa("Carrier")) {
			st = templates.getInstanceOf("instantiateCarrier");
		}
		else if (type.isa("Queue")) {
			st = templates.getInstanceOf("instantiateQueue");
		}

		st.setAttribute("direction", dir);
		st.setAttribute("var", fetch);

		return st;
	} 

	protected void constructGroup(StringTemplate code, boolean add,
		StringTemplate attr, StringTemplate temp, String direction) {

		StringTemplate assigned = null;
		StringTemplate last = null;
		String fetchAttr = null;
		String typeName = null;
		
		if (attr != null) {
			last = lastValueOf(attr, "index");
			typeName = (String) last.getAttribute("type");
			TypeInfo type = grammar.getType(typeName);
			if (type.isa("Payload")) {
				assigned = templates.getInstanceOf("createAST");
			}
			else {
				assigned = templates.getInstanceOf("createNew");
				assigned.setAttribute("type", typeName);
			}

			assigned.setAttribute("args", temp);

			if (add) {
				fetchAttr = attr.toString();
				StringTemplate tmp = 
					addAST(fetchAttr, direction, type);
				code.setAttribute("addAST", tmp);
			}
			last.setAttribute("value", assigned);

			code.setAttribute("assign", attr);
		}
		else {
			assigned = templates.getInstanceOf("createAST");
			assigned.setAttribute("args", temp);
			assigned.setAttribute("type", typeName);

			last = templates.getInstanceOf("instantiatePayload");
			last.setAttribute("direction", direction);
			last.setAttribute("var", assigned);

			code.setAttribute("addAST", last);
		}	
	} 

	protected String baseType(String type) {
		TypeInfo info = grammar.getTypes().getInfo(type);
		return info.getBaseType();
	}

	protected String strText(GrammarAST s) {
		String str = s.getText();
		return str.substring(1, str.length() -1);
	}

	protected StringTemplate strTemplate(GrammarAST s) {
		String str = strText(s);
		StringTemplate retval = templates.getInstanceOf("targetString");
		retval.setAttribute("str", str);
		return retval;
	}

	protected String findType(String name) {
		TypeInfo info = grammar.getTypes().getInfo(name);

		return info.getName();
	}

	protected void setTypeInfo(String parent, String name, StringTemplate tplt) {
		TypeInfo info = grammar.getTypes().getInfo(parent);
		String type = info.getName();
		String creator = "create" + info.getBaseType();

		tplt.setAttribute("type", type);
		if (templates.isDefined("create" + info.getBaseType())) {
			StringTemplate cr = templates.getInstanceOf(creator);
			cr.setAttribute("type", name);
			tplt.setAttribute("creators", cr);
		}
		else {
			StringTemplate cr = templates.getInstanceOf("defaultCreator");
			cr.setAttribute("type", name);
			tplt.setAttribute("creators", cr);
		}
		tplt.setAttribute("name", name); 
	}
}



file[Grammar g,
        StringTemplate rST,
        StringTemplate oST,
        StringTemplate hST]
	:
	{	init(g);
		this.recognizerST = rST;
		this.outputFileST = oST;
		this.headerFileST = hST;
	}
	(
		fileheader
	)*
	(
		typeDecls
	)?
	( optionsSpec )?
	grammar[g]
	;


fileheader
	:
	#(	"header"
		( ID )?
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
	)
	;


optionsSpec 
	:
	#( OPTIONS (option)+ )
	;

option
	:
	#( ASSIGN ID optionValue )
	;

optionValue
	:
	ID
	|	STRING_LITERAL
	|	KEYWORD
	|	INT
	;

// Attribute specification syntax
grammarAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			(
				grammarAttributeDecl
			)+
			{ recognizerST.setAttribute("variables", variables); }
		)?
	)
	;

grammarAttributeDecl
{	StringTemplate decl = null;
	StringTemplate visibility = null;
}
	:
	(
		"public"
		{ visibility = vpublic; }
	)?
	attributeVarDecl[variables, visibility]
	|
		"import"
		STRING_LITERAL
	;

ruleAttributeDecls
returns [ StringTemplate[] codes = null ]
{ StringTemplate acts[] = null; }
	:
	#( 
		ATTRIBUTE_STMT
		(
			{	if ( codes == null) {
					codes = new StringTemplate[3];
					codes[0] = templates.getInstanceOf("stmtList");
					codes[1] = templates.getInstanceOf("stmtList");
					codes[2] = templates.getInstanceOf("stmtList");
				}
			}
			acts = attributeUseDecl
			{	codes[0].setAttribute("body", acts[0]); 
				if (acts[1] != null)
					codes[1].setAttribute("body", acts[1]);

				codes[2].setAttribute("body", acts[2]);
			}
			|
				attributeVarDecl[variables, null]
		)*
	)
	;


typeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeTypeDecl
		)*
	)
	;
	

attributeTypeDecl
{	StringTemplate atype = templates.getInstanceOf("classDecl");
	boolean isNative = false;
}
	:
	#( 
		"atomic"
		"native"
		ID
	)
	|
		#( 
			LCURLY
			(
				"native"
				{ isNative = true; }
			)?
			t:ID
			n:ID
			{ setTypeInfo(#t.getText(), #n.getText(), atype); }
			(
				attributeVarDecl[atype, vpublic]
			)*
			{	if (!isNative)
					generator.addType(#n.getText(), atype);
			}
		)
	;

attributeVarDecl
[ StringTemplate av, StringTemplate visibility ]
{	StringTemplate v = templates.getInstanceOf("varDecl");
	StringTemplate a = templates.getInstanceOf("varAccessors");
	StringTemplate gType = null;
}
	:
	#( 
		SEMI
		t:ID
		(	OPEN_ELEMENT_OPTION
			i1:ID 
			{	gType = templates.getInstanceOf("genericType");
				gType.setAttribute("type", findType(#t.getText()));
				gType.setAttribute("genericArgs", #i1.getText()); }
			(	i2:ID
				{ gType.setAttribute("genericArgs", #i2.getText());}
			)*
			CLOSE_ELEMENT_OPTION
		)?
		n:ID
		{	if (gType != null) {
				v.setAttribute("type", gType);
				a.setAttribute("type", gType);
			}
			else {
				v.setAttribute("type", findType(#t.getText()));
				a.setAttribute("type", findType(#t.getText()));
			}
			a.setAttribute("basetype", baseType(#t.getText()));
			a.setAttribute("visibility", visibility);
			v.setAttribute("name", #n.getText());
			setAccessorNames(a, #n.getText());
		}
	)
	{	av.setAttribute("decls", v);
		av.setAttribute("accessors", a);
	}
	;


idQualifier[TypeInfo fieldType]
returns [ StringTemplate code = null ]
{	StringTemplate tail = null;
	StringTemplate index = null;
	String name = null;
	TypeInfo type = null;
	String typeName = null;
}
	:
	d:DOT_TEXT
	{	name = #d.getText().substring(1);
		typeName = fieldType.getTypeOf(name);
		type = grammar.getType(typeName);
	}
	(	code = arrayQualifier[name, type]
	|
		{	code = templates.getInstanceOf("Attribute");
			code.setAttribute("name", getUname(name));
			code.setAttribute("type", type.getName());
		}
	)
	;


arrayQualifier[String name, TypeInfo fieldType]
returns [ StringTemplate code = null ]
{	StringTemplate index = null;
	String typeID = null;
}
	:
	#(	LBRACKET
		{	code = templates.getInstanceOf("TableEntry");
			code.setAttribute("name", getUname(name));
			typeID = fieldType.getTypeOf("intrinsic.value");
			code.setAttribute("type", typeID);
		}		
		(	s:STRING_LITERAL
			{ code.setAttribute("contents", strTemplate(#s)); }
			|
				index = qualifiedId
				{ code.setAttribute("contents", index); }
		)
	)
	;


qualifiedId
returns [StringTemplate index = null; ]
{	StringTemplate tail = null;
	TypeInfo type = null;
	String name = null;
	String uname = null;
	String typeName = null;
}
	:
	i:ID
	{	index = templates.getInstanceOf("qualifiedAttribute");
		name = #i.getText().substring(1);
		typeName = grammar.getTypeOf(name);
		type = grammar.getTypeInfoFor(name);
		uname = getUname(name);
	}
	(	tail = arrayQualifier[name, type]
		|
			{	tail = templates.getInstanceOf("Attribute");
				tail.setAttribute("name", uname);
				tail.setAttribute("type", type.getName());
			}
	)
	{ index.setAttribute("index", tail); }
	(	tail = id:idQualifier[type]
		{	index.setAttribute("index", tail);
			String typeID = type.getTypeOf(#id.getText().substring(1));
			type = grammar.getType(typeID);
		}
	)*
	{ index.setAttribute("type", tail.getAttribute("type")); }
	;


attributeUseDecl
returns [ StringTemplate[] codes = new StringTemplate[3] ]
{	StringTemplate val = null;
	StringTemplate attr = null;
	StringTemplate temp = null;
}
	:
	v:ID
	{	temp = templates.getInstanceOf("useAttribute");
		temp.setAttribute("name", #v.getText());
		temp.setAttribute("rule", currentRuleName);
		temp.setAttribute("type", grammar.getTypeOf(#v.getText()));
		codes[0] = temp;

		attr = templates.getInstanceOf("AssignAttributeValue");
		attr.setAttribute("name", getUname(#v.getText()));
		attr.setAttribute("type", grammar.getTypeOf(#v.getText()));
	}
	ASSIGN
	val = qualifiedId
	{	attr.setAttribute("value", val);
		codes[1] = attr;

		temp = templates.getInstanceOf("restoreAttribute");
		temp.setAttribute("name", #v.getText());
		temp.setAttribute("rule", currentRuleName);
		codes[2] = temp;
	}
	|
		"new"
		n:ID
		{	temp = templates.getInstanceOf("useAttribute");
			temp.setAttribute("name", #n.getText());
			temp.setAttribute("rule", currentRuleName);
			temp.setAttribute("type", grammar.getTypeOf(#n.getText()));
			codes[0] = temp;

			attr = templates.getInstanceOf("AssignAttributeValue");
			attr.setAttribute("name", getUname(#n.getText()));
			StringTemplate value = templates.getInstanceOf("createNew");
			String type = grammar.getTypeOf(#n.getText());
			value.setAttribute("type", type);
			attr.setAttribute("value", value);
			codes[1] = attr;
	
			temp = templates.getInstanceOf("restoreAttribute");
			temp.setAttribute("name", #n.getText());
			temp.setAttribute("rule", currentRuleName);
			codes[2] = temp;
		}
	|
		"using"
		u:ID
		{	temp = templates.getInstanceOf("useAttribute");
			temp.setAttribute("name", #u.getText());
			temp.setAttribute("rule", currentRuleName);
			temp.setAttribute("type", grammar.getTypeOf(#u.getText()));
			codes[0] = temp;
			codes[1] = null;
	
			temp = templates.getInstanceOf("restoreAttribute");
			temp.setAttribute("name", #u.getText());
			temp.setAttribute("rule", currentRuleName);
			codes[2] = temp;
		}
	;
// END ATTRIBUTE DECLARATION and TYPE DEFINITION


grammar[Grammar g]
{
    String superClass = (String)g.getOption("superClass");
    outputOption = (String)g.getOption("output");
    recognizerST.setAttribute("superClass", superClass);
    if ( g.type!=Grammar.LEXER ) {
		recognizerST.setAttribute("ASTLabelType", g.getOption("ASTLabelType"));
	}
    if ( g.type!=Grammar.TREE_PARSER ) {
		recognizerST.setAttribute("labelType", g.getOption("TokenLabelType"));
	}
	recognizerST.setAttribute("numRules", grammar.getRules().size());
	outputFileST.setAttribute("numRules", grammar.getRules().size());
	headerFileST.setAttribute("numRules", grammar.getRules().size());
}
    :   ( #( LEXER_GRAMMAR grammarSpec )
	    | #( PARSER_GRAMMAR grammarSpec )
	    | #( TREE_GRAMMAR grammarSpec
	       )
	    | #( COMBINED_GRAMMAR grammarSpec )
	    )
    ;


grammarSpec
	:
	name:ID
	(cmt:DOC_COMMENT {outputFileST.setAttribute("docComment", #cmt.getText());} )?
	{
		recognizerST.setAttribute("name", #name.getText());
		outputFileST.setAttribute("name", #name.getText());
		headerFileST.setAttribute("name", #name.getText());
	}
	(
		grammarAttributeDecls
	)?
		( optionsSpec )?
		( tokensSpec )?
		rules[recognizerST]
	;


tokensSpec
{ StringTemplate spec = null; }
	:
	#(	TOKENS
		(	spec = tokenSpec
			{	if (spec != null)
					factoryFileST.setAttribute("cases", spec);
			}
		)+
	)
	;


tokenSpec
returns [ StringTemplate spec = null ]
	:
	t:TOKEN_REF
	(	OPEN_ELEMENT_OPTION
		i:ID
		{	spec = templates.getInstanceOf("buildCase");
			spec.setAttribute("tokenType", #t.getText());
			spec.setAttribute("tokenClass", #i.getText());
		}
	)?
	|
		#(	ASSIGN
			t2:TOKEN_REF 
			(	OPEN_ELEMENT_OPTION
				i2:ID
				{	spec = templates.getInstanceOf("buildCase");
					spec.setAttribute("tokenType", #t2.getText());
					spec.setAttribute("tokenClass", #i2.getText());
		}
			)?
			c:KEYWORD
		 )
	;


rules[StringTemplate recognizerST]
{
StringTemplate rST;
StringTemplate cST;
}
	:
	(	rST=rule
    		{
	    		if ( rST!=null ) {
				recognizerST.setAttribute("rules", rST);
				outputFileST.setAttribute("rules", rST);
				headerFileST.setAttribute("rules", rST);
			}
    		}
		|
			cST=constructorRule
    			{
	    			if ( cST!=null ) {
					recognizerST.setAttribute("crules", cST);
					outputFileST.setAttribute("crules", cST);
					headerFileST.setAttribute("crules", cST);
				}
    			}
    		|
			RULE
	)+
    ;

rule
returns [StringTemplate code=null]
{
	String r = null;
	String initAction = null;
	StringTemplate b;
  	StringTemplate[] decls = null;
  	StringTemplate act = null;
	// get the dfa for the BLOCK
	GrammarAST block=#rule.getFirstChildWithType(BLOCK);
	DFA dfa=block.getLookaheadDFA();
	// init blockNestingLevel so it's block level RULE_BLOCK_NESTING_LEVEL
	// for alts of rule
	blockNestingLevel = RULE_BLOCK_NESTING_LEVEL-1;
	Rule ruleDescr = grammar.getRule(#rule.getFirstChild().getText());

	// For syn preds, we don't want any AST code etc... in there.
	// Save old templates ptr and restore later.  Base templates include Dbg.
	if ( ruleDescr.isSynPred ) {
		inSynPred = true;
	}
}
    :   #( RULE id:ID {r=#id.getText(); currentRuleName = r;}
		(mod:modifier)?
		( optionsSpec )?
		( decls = ruleAttributeDecls )?
		( act = action )?
	     	b=block["ruleBlock", dfa]

		{
			String description =
				grammar.grammarTreeToString(#rule.getFirstChildWithType(BLOCK), false);
			description = generator.target.getTargetStringLiteralFromString(description);
			b.setAttribute("description", description);
			/*
			System.out.println("rule "+r+" tokens="+
							   grammar.getRule(r).getAllTokenRefsInAltsWithRewrites());
			System.out.println("rule "+r+" rules="+
							   grammar.getRule(r).getAllRuleRefsInAltsWithRewrites());
			*/
			// do not generate lexer rules in combined grammar
			String stName = null;
			if ( ruleDescr.isSynPred ) {
				stName = "synpredRule";
			}
			else if ( grammar.type==Grammar.LEXER ) {
				if ( r.equals(Grammar.ARTIFICIAL_TOKENS_RULENAME) ) {
					stName = "tokensRule";
				}
				else {
					stName = "lexerRule";
				}
			}
			else {
				if ( !(grammar.type==Grammar.COMBINED &&
					 Character.isUpperCase(r.charAt(0))) )
				{
					stName = "rule";
				}
			}
			code = templates.getInstanceOf(stName);
			if ( code.getName().equals("rule") ) {
				code.setAttribute("emptyRule",
					new Boolean(grammar.isEmptyRule(block)));
			}
			code.setAttribute("ruleDescriptor", ruleDescr);
			String memo = (String)#rule.getOption("memoize");
			if ( memo==null ) {
				memo = (String)grammar.getOption("memoize");
			}
			if ( memo!=null && memo.equals("true") &&
			     (stName.equals("rule")||stName.equals("lexerRule")) )
			{
		            	code.setAttribute("memoize",
            			new Boolean(memo!=null && memo.equals("true")));
            		}
			if (act != null)
				code.setAttribute("init", act);
			if (decls != null) {
				code.setAttribute("save", decls[0]);
				code.setAttribute("init", decls[1]);
				code.setAttribute("restore", decls[2]);
			}
		}

	     	(exceptionGroup[code])?
	     	EOR
         )
        {
        if ( code!=null ) {
			if ( grammar.type==Grammar.LEXER ) {
		    	boolean naked =
		    		r.equals(Grammar.ARTIFICIAL_TOKENS_RULENAME) ||
		    	    (mod!=null&&mod.getText().equals(Grammar.FRAGMENT_RULE_MODIFIER));
		    	code.setAttribute("nakedBlock", new Boolean(naked));
			}
			else {
				String description =
					grammar.grammarTreeToString(#rule,false);
				code.setAttribute("description", description);
			}
			Rule theRule = grammar.getRule(r);
			code.setAttribute("ruleName", r);
			code.setAttribute("block", b);
			if ( initAction!=null ) {
				code.setAttribute("initAction", initAction);
			}
        	}
		inSynPred = false;
        }
    ;


constructorRule
returns [StringTemplate code=null]
	:
	#(	ATTRIBUTE_COLON
		RULE_REF
		( attributeItem )+
	)
	;


modifier
	:	"protected"
	|	"public"
	|	"private"
	|	"fragment"
	;


block[String blockTemplateName, DFA dfa]
     returns [StringTemplate code=null]
{
    StringTemplate decision = null;
    if ( dfa!=null ) {
        code = templates.getInstanceOf(blockTemplateName);
        decision = generator.genLookaheadDecision(recognizerST,dfa);
        code.setAttribute("decision", decision);
        code.setAttribute("decisionNumber", dfa.getDecisionNumber());
		code.setAttribute("maxK",dfa.getMaxLookaheadDepth());
		code.setAttribute("maxAlt",dfa.getNumberOfAlts());
    }
    else {
        code = templates.getInstanceOf(blockTemplateName+"SingleAlt");
    }
    blockNestingLevel++;
    code.setAttribute("blockLevel", blockNestingLevel);
    code.setAttribute("enclosingBlockLevel", blockNestingLevel-1);
    StringTemplate alt = null;
    GrammarAST r = null;
    int altNum = 1;
    if ( this.blockNestingLevel==RULE_BLOCK_NESTING_LEVEL )
	{this.outerAltNum=1;}
}
    :   #(  BLOCK
    	    ( optionsSpec )? // ignore
            ( alt=alternative {r=(GrammarAST)_t;} 
              {
              if ( this.blockNestingLevel==RULE_BLOCK_NESTING_LEVEL ) {
              	this.outerAltNum++;
              }
    		  // add this alt to the list of alts for this block
              code.setAttribute("alts",alt);
              alt.setAttribute("altNum", new Integer(altNum));
              alt.setAttribute("outerAlt",
                  new Boolean(blockNestingLevel==RULE_BLOCK_NESTING_LEVEL));
              altNum++;
              }
            )+
            EOB
         )
    	{blockNestingLevel--;}
    ;

exceptionGroup[StringTemplate ruleST]
	:	( exceptionSpec[ruleST] )+  // we only handle one group now
    ;

exceptionSpec[StringTemplate ruleST]
    :   #("exception" ( arg_decl )? ( exceptionHandler[ruleST] )*)
    ;


arg_decl
returns [ StringTemplate code = null ]
	:
	#(	RBRACKET
		code = template
	)
	;
	

exceptionHandler[StringTemplate ruleST]
{	StringTemplate act = null;
	StringTemplate arg = null;
}
    :    #("catch" arg = arg_decl act = action)
    	{ ruleST.setAttribute("exceptions.{decl,action}", arg, act); }
    ;

alternative returns [StringTemplate code=templates.getInstanceOf("alt")]
{
String description = grammar.grammarTreeToString(#alternative, false);
description = generator.target.getTargetStringLiteralFromString(description);
code.setAttribute("description", description);
StringTemplate e;
GrammarAST elAST = null;
}
    :   #(	a:ALT
    		(	{elAST=(GrammarAST)_t;}
    			e=element
    			{
				if (e != null)
		    			code.setAttribute(
						"elements.{el,line,pos}",
    						e,
    						new Integer(elAST.getLine()),
    						new Integer(elAST.getColumn())
    					);
    			}
    		)+
    		EOA
    	 )
    ;


element
returns [StringTemplate code=null]
{	StringTemplate instantiate = null;
	StringTemplate temp = null;

	IntSet elements=null;
}
	:
	code=at:atom
	(	CARET
		{
			instantiate = templates.getInstanceOf("instantiateToken");
		 	instantiate.setAttribute("direction", "ROOT");
			code.setAttribute("instantiate", instantiate);
		}
		|	BANG
     			{	if ( grammar.type==Grammar.LEXER ) {
					code.setAttribute("skip", "true");
				}
			}
		|
		{	
     			if ( grammar.type!=Grammar.LEXER ) {
				instantiate = templates.getInstanceOf("instantiateToken");
		 		instantiate.setAttribute("direction", "RIGHT");
				code.setAttribute("instantiate", instantiate);
			}
		}
	)
	|	code = ruleReference
		(	BANG
			{ 	temp = templates.getInstanceOf("ASTDisable");
				code.setAttribute("before", temp);
				temp = templates.getInstanceOf("ASTRevert");
				code.setAttribute("after", temp);
			}
		)?
	|   #(CHAR_RANGE a:KEYWORD b:KEYWORD)
        {code = templates.getInstanceOf("charRangeRef");
		 String low =
		 	generator.target.getTargetCharLiteralFromANTLRCharLiteral(generator,a.getText());
		 String high =
		 	generator.target.getTargetCharLiteralFromANTLRCharLiteral(generator,b.getText());
         code.setAttribute("a", low);
         code.setAttribute("b", high);
        }

    |   (labeledItem)=>
	code = labeledItem
    |	code = attributeItem // instantiate
    |   code=ebnf
    |   code=treeMatcher
	(	BANG
		{	StringTemplate e = code;
			code = templates.getInstanceOf("statements");
			temp = templates.getInstanceOf("ASTDisable");
			code.setAttribute("body", temp);
			code.setAttribute("body", e);
			temp = templates.getInstanceOf("ASTRevert");
			code.setAttribute("body", temp);
		}
	)?

    |   temp = action
	{	if (!inSynPred) {
			code = templates.getInstanceOf("execAction");
			code.setAttribute("action", temp);
		}
	}
    |   code = sempred
    |   sp:GATED_SEMPRED
        {
        code = templates.getInstanceOf("validateSemanticPredicate");
        #sp.outerAltNum = this.outerAltNum;
        code.setAttribute("pred", #sp.getText());
		String description =
			generator.target.getTargetStringLiteralFromString(#sp.getText());
		code.setAttribute("description", description);
        }

    |	SYN_SEMPRED // used only in lookahead; don't generate validating pred

    |   EPSILON
    ;


negatedAtom
returns [ StringTemplate code = null ]
{	IntSet elements=null;
	StringTemplate instantiate = null;
}
	:
	#(	n:NOT
         	(   c:KEYWORD
	           {
	            int ttype=0;
     			if ( grammar.type==Grammar.LEXER ) {
        			ttype = Grammar.getCharValueFromGrammarCharLiteral(c.getText());
     			}
     			else {
        			ttype = grammar.getTokenType(#c.getText());
        		}
	            elements = grammar.complement(ttype);
	           }
            |   s:STRING_LITERAL
	           {
	            int ttype=0;
     			if ( grammar.type==Grammar.LEXER ) {
        			// TODO: error!
     			}
     			else {
        			ttype = grammar.getTokenType(strText(#s));
        		}
	            elements = grammar.complement(ttype);
	           }
            |   t:TOKEN_REF
	           {
	           int ttype = grammar.getTokenType(t.getText());
	           elements = grammar.complement(ttype);
	           }
            |   #( st:SET (setElement)+ )
               {
               // SETs are not precomplemented by buildnfa.g like
               // simple elements.
               elements = st.getSetValue();
               }
            )
            {
		if ( grammar.type==Grammar.LEXER ) {
			code = templates.getInstanceOf("matchCharSet");
		}
		else {
            		code = templates.getInstanceOf("matchSet");
		 	code.setAttribute("elementIndex", ((TokenWithIndex)#n.getToken()).getIndex());
		 	generator.generateLocalFOLLOW(#n,"set",currentRuleName);
        	}
            	code.setAttribute("s", generator.genSetExpr(templates,elements,1,false));
            }
         )
	;


labeledItem
returns [StringTemplate code=null]
{	StringTemplate inst = null;
	StringTemplate stTok = null;
	StringTemplate temp = null;
	String dir = "RIGHT";
	StringTemplate last = null;
	StringTemplate getTok = null;
	StringTemplate temp2 = null;
}
	:
	(	stTok = attrLabel		
		{ last = lastValueOf(stTok, "index"); }
		(	code=atom
     			{	if ( grammar.type==Grammar.LEXER ) {
					// leave blank for now
				}
				else {
					getTok = templates.getInstanceOf("lastPayload");
					last.setAttribute("value", getTok);
					code.setAttribute("instantiate", stTok);
				}
			}
		|	code = ruleReference
			{	temp = templates.getInstanceOf("ASTEnable");
				code.setAttribute("before", temp);
				temp = templates.getInstanceOf("ASTRevert");
				code.setAttribute("after", temp);
				getTok = templates.getInstanceOf("getCarrier");
				last.setAttribute("value", getTok);
				code.setAttribute("instantiate", stTok);
			}
		|
			(	temp = baseebnf
				|
					temp = treeMatcher
			)
			{	code = templates.getInstanceOf("ebnfStmts");
				temp2 = templates.getInstanceOf("preCarrierBlock");
				code.setAttribute("body", temp2);
				code.setAttribute("body", temp);
				temp2 = templates.getInstanceOf("postCarrierBlock");
				code.setAttribute("body", temp2);
				getTok = templates.getInstanceOf("getCarrier");
				last.setAttribute("value", getTok);
				code.setAttribute("body", stTok);
			}
	)
	|	stTok = attrAssignInstance
		{ last = lastValueOf(stTok, "index"); }
		(	code=atom
			{	inst = templates.getInstanceOf("instantiateToken");
				code.setAttribute("instantiate", inst);
				getTok = templates.getInstanceOf("lastPayload");
				last.setAttribute("value", getTok);
				code.setAttribute("instantiate", stTok);
			}
			(	CARET
				{ inst.setAttribute("direction", "ROOT"); }
				|
					{ inst.setAttribute("direction", "RIGHT"); }
			)
		|	code = ruleReference
			{	temp = templates.getInstanceOf("ASTEnable");
				code.setAttribute("before", temp);
				temp = templates.getInstanceOf("ASTRevert");
				code.setAttribute("after", temp);
				getTok = templates.getInstanceOf("lastCarrier");
				code.setAttribute("instantiate", stTok);
			}
		|
			(	temp = baseebnf
				|
					temp = treeMatcher
			) 
			{	code = templates.getInstanceOf("ebnfStmts");
				temp2 = templates.getInstanceOf("preCarrierBlock");
				code.setAttribute("body", temp2);
				code.setAttribute("body", temp);
				temp = templates.getInstanceOf("postCarrierBlock");
				code.setAttribute("body", temp);
				getTok = templates.getInstanceOf("lastCarrier");
				last.setAttribute("value", getTok);
				code.setAttribute("body", stTok);
				temp = templates.getInstanceOf("removeExcessCarrier");
				code.setAttribute("body", temp);
			}
		)

	)
	;


// Templated actions
action
returns [ StringTemplate code = null ]
{ StringTemplate act = null; }
	:
	#(	RCURLY
		code = template
	)
	;


sempred
returns [ StringTemplate code = null ]
{ StringTemplate pred = null; }
	:
	#(	s:SEMPRED
		pred = template
		{	code = templates.getInstanceOf("validateSemanticPredicate");
		        code.setAttribute("pred", pred);
		}
	)
	;


predicatedConstructor
returns [ StringTemplate code = null ]
{ StringTemplate pred = null; }
	:
	#(	SEMPRED
		pred = template
		{	code = templates.getInstanceOf("constructionPred");
		        code.setAttribute("pred", pred);
		}
	)
	;


template
returns [ StringTemplate code = null ]
{	StringTemplate attr = null;
	String str = null;
}
	:
	(	s:STRING_LITERAL		// anonymous StringTemplate
		{ code = new StringTemplate(userTemplates, strText(#s)); }
		|
			i:ID		// StringTemplate name
			{ code = userTemplates.getInstanceOf(#i.getText()); }
	)
	(
		#(	v:ID
			(	attr = attribute
				{ code.setAttribute(#v.getText(), attr); }
			|
				(	t:TOKEN_REF { str = #t.getText(); }
					|
						s1:STRING_LITERAL
						{ str = strText(#s1); }
				)
				{ code.setAttribute(#v.getText(), str); }
			)
		)
	)*
	;



attrLabel
returns [ StringTemplate index = null ]
{	StringTemplate tail = null;
	TypeInfo type = null;
	String name = null;
	String uname = null;
	String typeName = null;
}
	:
	// assign and remove AST
	i:ASSIGN_ATTRIBUTE
	{	index = templates.getInstanceOf("qualifiedAttribute"); 
		name = #i.getText().substring(1);
		typeName = grammar.getTypeOf(name);
		type = grammar.getTypeInfoFor(name);
		uname = getUname(name);
	}
	(	tail = arrayQualifier[name, type]
		|
			{	tail = templates.getInstanceOf("Attribute");
				tail.setAttribute("name", uname);
				tail.setAttribute("type", type.getName());
			}
	)
	{ index.setAttribute("index", tail); }
	(	tail = id:idQualifier[type]
		{	index.setAttribute("index", tail);
			String typeID = type.getTypeOf(#id.getText().substring(1));
			type = grammar.getType(typeID);
		}
	)*
	;

attrAssignInstance
returns [ StringTemplate index = null ]
{	StringTemplate tail = null;
	TypeInfo type = null;
	String typeName = null;
	String name = null;
}
	:
	// assign but do not remove AST
	i:LABEL_ATTRIBUTE
	{	index = templates.getInstanceOf("qualifiedAttribute");
		String s = getUname(#i.getText());
		tail = templates.getInstanceOf("Attribute");
		tail.setAttribute("name", s);
		name = #i.getText();
		typeName = grammar.getTypeOf(name);
		type = grammar.getTypeInfoFor(name);
		tail.setAttribute("type", type.getName());
		index.setAttribute("index", tail);
	}
	(	tail = id:idQualifier[type]
		{	index.setAttribute("index", tail);
			String typeID = type.getTypeOf(#id.getText().substring(1));
			type = grammar.getType(typeID);
		}
	)*
	;


ebnf returns [StringTemplate code = null]
{ StringTemplate temp = null; }
	:
	#(	BANGEDUP
		{	if (grammar.type == Grammar.LEXER)
				killChars = true;
		}
		code = baseebnf
		{	StringTemplate e = code;
			if (grammar.type == Grammar.LEXER)
				killChars = false;
			else {
				code = templates.getInstanceOf("statements");
				temp = templates.getInstanceOf("ASTDisable");
				code.setAttribute("body", temp);
				code.setAttribute("body", e);
				temp = templates.getInstanceOf("ASTRevert");
				code.setAttribute("body", temp);
			}
		}	
	)
	|
		code = baseebnf
	;


baseebnf returns [StringTemplate code=null]
{
    DFA dfa=null;
    GrammarAST b = (GrammarAST)#baseebnf.getFirstChild();
    GrammarAST eob = (GrammarAST)#b.getLastChild(); // loops will use EOB DFA
}
    :   (	{dfa = #baseebnf.getLookaheadDFA();}
			code=block["block", dfa]
		|   {dfa = #baseebnf.getLookaheadDFA();}
			#( OPTIONAL code=block["optionalBlock", dfa] )
		|   {dfa = #eob.getLookaheadDFA();}
			#( CLOSURE code=block["closureBlock", dfa] )
		|   {dfa = #eob.getLookaheadDFA();}
			#( POSITIVE_CLOSURE code=block["positiveClosureBlock", dfa] )
		)
		{
		String description = grammar.grammarTreeToString(#baseebnf, false);
		description = generator.target.getTargetStringLiteralFromString(description);
    	code.setAttribute("description", description);
    	}
    ;


treeMatcher
returns [StringTemplate code=templates.getInstanceOf("tree")]
{
StringTemplate el=null;
GrammarAST elAST=null;
boolean oldIsTree = isTree;
isTree = true;
}
	:
	#(	TREE_BEGIN {elAST=(GrammarAST)_t;}
    		el=rootElement
        	{	code.setAttribute("root", el); }
		(
			{elAST=(GrammarAST)_t;}
    			el=element
           		{	if (el != null)
					code.setAttribute("children", el);
			}
		)+
		{	if (isTree) {
				el = templates.getInstanceOf("makeNextUp");
				code.setAttribute("children", el);
			}
		}
	)
	{ isTree = oldIsTree; }
	;


rootElement
returns [ StringTemplate code = null ]
{	boolean addToTree = addingAttributes;
	StringTemplate temp = null;
	StringTemplate instantiate = null;
	StringTemplate last = null;
	StringTemplate attr = null;
}
	:
	(	attr = attrLabel
		{	addToTree = false;
		}
	|
		attr = attrAssignInstance
	)?
	temp = atom
	{
		if (addToTree) {
			instantiate = templates.getInstanceOf("instantiateRootToken");
			temp.setAttribute("instantiate", instantiate);
			if (attr == null)
				code = temp;
		}
		else
			isTree = false;

		if (attr != null) {
			last = lastValueOf(attr, "index");
			code = attr;
			instantiate = templates.getInstanceOf("lastPayload");
			last.setAttribute("value", instantiate);
			code = templates.getInstanceOf("statements");
			code.setAttribute("body", temp);
			code.setAttribute("body", attr);
		}
	}
	;



attribute
returns [StringTemplate index = null; ]
{	StringTemplate tail = null;
	String name = null;
	TypeInfo type = null;
	String typeName = null;
}
	:
	a:ATTRIBUTE
	{	index = templates.getInstanceOf("qualifiedAttribute");
		name = #a.getText().substring(1);
		typeName = grammar.getTypeOf(name);
		type = grammar.getTypeInfoFor(name);
	}
	(	tail = arrayQualifier[name, type]
		|
			{	tail = templates.getInstanceOf("Attribute");
				tail.setAttribute("name", getUname(name));
				tail.setAttribute("type", type.getName());
			}
	)
	{ index.setAttribute("index", tail); }
	(	tail = id:idQualifier[type]
		{	index.setAttribute("index", tail);
			String typeID = type.getTypeOf(#id.getText().substring(1));
			type = grammar.getType(typeID);
		}
	)*
	;


ruleReference
returns [StringTemplate code = null]
	:
	r:RULE_REF
        {
        grammar.checkRuleReference(#r, null, currentRuleName);
        code = templates.getInstanceOf("ruleRef");
		code.setAttribute("rule", r.getText());

		code.setAttribute("elementIndex", ((TokenWithIndex)r.getToken()).getIndex());
		generator.generateLocalFOLLOW(#r,#r.getText(),currentRuleName);
		#r.code = code;
        }

	;


atom
returns [StringTemplate code=null]
    :
        t:TOKEN_REF
        {
           grammar.checkRuleReference(#t, null, currentRuleName);
		   if ( grammar.type==Grammar.LEXER ) {
			    if ( grammar.getTokenType(t.getText())==Label.EOF ) { code = templates.getInstanceOf("lexerMatchEOF");
				}
			    else {
					code = templates.getInstanceOf("lexerRuleRef");
					code.setAttribute("rule", t.getText());
				}
		   }
		   else {
			   code = templates.getInstanceOf("tokenRef");
			   code.setAttribute("token", t.getText());
			   code.setAttribute("elementIndex", ((TokenWithIndex)#t.getToken()).getIndex());
			   generator.generateLocalFOLLOW(#t,#t.getText(),currentRuleName);
		   }
		   #t.code = code;
		}

    |   c:KEYWORD
        {
		if ( grammar.type==Grammar.LEXER ) {
			code = templates.getInstanceOf("charRef");
			code.setAttribute("char",
			   generator.target.getTargetCharLiteralFromANTLRCharLiteral(generator,c.getText()));
			if (killChars)
				code.setAttribute("skip", "true");
		}
		else { // else it's a token type reference
			code = templates.getInstanceOf("tokenRef");
			code.setAttribute("token",
							  new Integer(grammar.getTokenType(c.getText())));
			code.setAttribute("elementIndex",
							  ((TokenWithIndex)#c.getToken()).getIndex());
			generator.generateLocalFOLLOW(#c,
				String.valueOf(grammar.getTokenType(#c.getText())),
				currentRuleName);
		}
        }

    |   s:STRING_LITERAL
        {
		if ( grammar.type==Grammar.LEXER ) {
			code = templates.getInstanceOf("lexerStringRef");
			code.setAttribute("string",
			   generator.target.getTargetStringLiteralFromANTLRStringLiteral(generator,s.getText()));
			if (killChars)
				code.setAttribute("skip", "true");
		}
		else { // else it's a token type reference
			code = templates.getInstanceOf("tokenRef");
			code.setAttribute("token",
							 new Integer(grammar.getTokenType(s.getText())));
			code.setAttribute("elementIndex", ((TokenWithIndex)#s.getToken()).getIndex());
			generator.generateLocalFOLLOW(#s,
				String.valueOf(grammar.getTokenType(#s.getText())),
				currentRuleName);
		}
	}

    |   w:WILDCARD
        {
		code = getWildcardST(#w,null,null);
		code.setAttribute("elementIndex", ((TokenWithIndex)#w.getToken()).getIndex());
		}

    |	code=set[null]
    |   code = negatedAtom
    ;


set[String label]
returns [StringTemplate code=null]
{	StringTemplate instantiate = null; }
	:
	#( s:SET (setElement)+ )
        {
		if ( grammar.type==Grammar.LEXER ) {
			code = templates.getInstanceOf("matchCharSet");
			if (killChars) {
				code.setAttribute("skip", "true");
			}
		}
		else
		{	code = templates.getInstanceOf("matchSet");
			code.setAttribute("elementIndex", ((TokenWithIndex)#s.getToken()).getIndex());
			generator.generateLocalFOLLOW(#s,"set",currentRuleName);
        	}
	        code.setAttribute("s", generator.genSetExpr(templates,#s.getSetValue(),1,false));
        }
	;

setElement
    :   c:KEYWORD
    |   t:TOKEN_REF
    |   s:STRING_LITERAL
    |	#(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
    ;


// ATTRIBUTE syntax
attributeGroup
returns [ StringTemplate code = templates.getInstanceOf("arguments") ]
{	StringTemplate temp = null;
	boolean addState = addingAttributes;
}
	:
	{ addingAttributes = false; }
	#( 
		ATTRIBUTE_GROUP
		(
			temp = attributeItem
			{ code.setAttribute("body", temp); }
			|
				s:STRING_LITERAL
				{ code.setAttribute("body", strTemplate(s)); }
		)*
	)
	{ addingAttributes = addState; }
	;


attributeBlock
returns [StringTemplate code = null]
{ StringTemplate temp = null; }
	:
	{ code = templates.getInstanceOf("statements"); }
	(
		temp = attributeItem
		{ code.setAttribute("body", temp); }
	)*
	|
		code = predicatedConstructor
		(
			temp = attributeItem
			{ code.setAttribute("body", temp); }
		)*
		( 
			OR
			temp = attributeBlock
			{ code.setAttribute("altBody", temp); }
		)
	;


attributeStatements
returns [StringTemplate code = null]
	:
	#(	ATTRIBUTE_STMT
		code = attributeBlock
	)
	;


attributeItem
returns [StringTemplate code = null]
{	StringTemplate attr = null;
	boolean addToTree = addingAttributes;
}
	:
	(	attr = attrLabel
		{ addToTree = false; }
	|
		attr = attrAssignInstance
	)?
	code = attributeElement[addToTree, attr]
	;

attributeElement[ boolean add, StringTemplate attr ]
returns [ StringTemplate code = templates.getInstanceOf("instantiateAttribute");]
{	StringTemplate assigned = null;
	StringTemplate temp = null;
	StringTemplate last = null;
	String direction = "RIGHT";
}
	:
	assigned = attribute
	(
		CARET
		{ direction = "ROOT"; }
	)?
	{	if (add) {
			temp = addAST(assigned, direction);
			code.setAttribute("addAST", temp);
		}
		if (attr != null) {
			last = lastValueOf(attr, "index");
			last.setAttribute("value", assigned);
			code.setAttribute("assign", attr);
		}
		else if (!add) {
			code = assigned;
		}
	}
	|
		temp = attributeGroup
		(
			CARET
			{ direction = "ROOT"; }
		)?
		{ constructGroup(code, add, attr, temp, direction); }
	|
		code = attributeStatements
	|
		CURRENT_TREE
		(
			BANG
			{ code = templates.getInstanceOf("killTree"); }
			|
			{	code = templates.getInstanceOf("assignTree");
				temp = templates.getInstanceOf("getCarrier");
				attr.setAttribute("value", temp);
				code.setAttribute("instantiate", attr);
			}
		)
	|
		LPAREN
		{ code = templates.getInstanceOf("statements"); }
		(
			temp = attributeItem
			{ code.setAttribute("body", temp); }
		)*
		RPAREN
	|
		code = treeConstructor
	;


treeConstructor
returns [ StringTemplate code = templates.getInstanceOf("statements") ]
{ StringTemplate temp = null; }
	:
	#(
		TREE_CONSTRUCTOR
		temp = attributeItem
		{	code.setAttribute("body", temp);
			temp = templates.getInstanceOf("makeNextDown");
			code.setAttribute("body", temp);
		}
		(
			temp = attributeItem
			{ code.setAttribute("body", temp); }
		)+
		{	temp = templates.getInstanceOf("makeNextUp");
			code.setAttribute("body", temp);
		}
	)
	;
// END ATTRIBUTE syntax

