header {
/*
 [The "BSD licence"]
 Copyright (c) 2005-2006 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
	package org.antlr_yggdrasil.generated;
	import org.antlr_yggdrasil.tool.*;
	import java.util.*;
}

/** Print out a grammar (no pretty printing).
 *
 *  Terence Parr
 *  University of San Francisco
 *  August 19, 2003
 */
class YggdrasilTreePrinter extends TreeParser;

options {
	importVocab = Yggdrasil;
	ASTLabelType = "GrammarAST";
    codeGenBitsetTestThreshold=999;
}

{
	protected Grammar grammar;
	protected boolean showActions;
    protected StringBuffer buf = new StringBuffer(300);

    public void out(String s) {
        buf.append(s);
    }

    public void reportError(RecognitionException ex) {
		Token token = null;
		if ( ex instanceof MismatchedTokenException ) {
			token = ((MismatchedTokenException)ex).token;
		}
		else if ( ex instanceof NoViableAltException ) {
			token = ((NoViableAltException)ex).token;
		}
        ErrorManager.syntaxError(
            ErrorManager.MSG_SYNTAX_ERROR,
            token,
            "YggdrasilPrint: "+ex.toString(),
            ex);
    }

	/** Normalize a grammar print out by removing all double spaces
	 *  and trailing/beginning stuff.  FOr example, convert
	 *
	 *  ( A  |  B  |  C )*
	 *
	 *  to
	 *
	 *  ( A | B | C )*
	 */
	public static String normalize(String g) {
	    StringTokenizer st = new StringTokenizer(g, " ", false);
		StringBuffer buf = new StringBuffer();
		while ( st.hasMoreTokens() ) {
			String w = st.nextToken();
			buf.append(w);
			buf.append(" ");
		}
		return buf.toString().trim();
	}
}

/** Call this to figure out how to print */
toString[Grammar g, boolean showActions] returns [String s=null]
{
grammar = g;
this.showActions = showActions;
}
    :   (
	fileheader
	|	grammarAttributeDecls
	|	optionsSpec
	|	grammar
        |	rule
        |	alternative
        |	elementPrint
        |	EOR {s="EOR";}
        )
        {return normalize(buf.toString());}
    ;

// --------------

file
	:
	(
		fileheader
	)*
	(
		typeDecls
	)?
	(
		optionsSpec
	)?
	grammar
	;


fileheader
	:
	#(	"header"
		( ID )?
		LCURLY
		( optionsSpec )?
		( tokensSpec )?
		( action )?
	)
	;


// Attribute specification syntax
grammarAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			grammarAttributeDecl
		)*
	)
	;

grammarAttributeDecl
	:
	(
		"public"
	)?
	attributeVarDecl
	|
		"import"
		STRING_LITERAL
	;

ruleAttributeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeVarDecl
			|
				attributeUseDecl
		)*
	)
	;


typeDecls
	:
	#( 
		ATTRIBUTE_STMT
		(
			attributeTypeDecl
		)*
	)
	;
	

attributeTypeDecl
	:
	#( 
		"atomic"
		"native"
		ID
	)
	|
		#( 
			LCURLY
			(
				"native"
			)?
			ID
			ID
			(
				attributeVarDecl
			)*
		)
	;

attributeVarDecl
	:
	#( 
		SEMI
		ID
		(	OPEN_ELEMENT_OPTION
			( ID )+
			CLOSE_ELEMENT_OPTION
		)?
		ID
	)
	;


idQualifier
	:
	DOT_TEXT
	( arrayQualifier )?
	;

arrayQualifier
	:
	#(	LBRACKET
		(	STRING_LITERAL
			|
				ID
				( arrayQualifier )?
				(idQualifier)*
		)
	)
	;


attributeUseDecl
	:
	ID
	ASSIGN
	(
		ID
		( arrayQualifier )?
		( idQualifier )*
	)
	|
		"new"
		ID
	|
		"using"
		ID
	;
// END ATTRIBUTE DECLARATION and TYPE DEFINITION


grammar
    :   ( #( LEXER_GRAMMAR grammarSpec["lexer " ] )
	    | #( PARSER_GRAMMAR grammarSpec["parser "] )
	    | #( TREE_GRAMMAR grammarSpec["tree "] )
	    | #( COMBINED_GRAMMAR grammarSpec[""] )
	    )
     ;

grammarSpec[String gtype]
	:
	id:ID {out(gtype+"grammar "+#id.getText());}
        (cmt:DOC_COMMENT {out(#cmt.getText()+"\n");} )?
	(
		grammarAttributeDecls
	)?
        (optionsSpec)? {out(";\n");}
        (tokensSpec)?
        (action)?
        rules
    ;


optionsSpec
    :   #( OPTIONS {out(" options {");}
    	   (option {out("; ");})+
    	   {out("} ");}
    	 )
    ;

option
    :   #( ASSIGN id:ID {out(#id.getText()+"=");} optionValue )
    ;

optionValue
	:	id:ID            {out(#id.getText());}
	|	s:STRING_LITERAL {out(#s.getText());}
	|	c:KEYWORD   {out(#c.getText());}
	|	i:INT            {out(#i.getText());}
//	|   charSet
	;

/*
charSet
	:   #( CHARSET charSetElement )
	;

charSetElement
	:   c:KEYWORD {out(#c.getText());}
	|   #( OR c1:KEYWORD c2:KEYWORD )
	|   #( RANGE c3:KEYWORD c4:KEYWORD )
	;
*/

tokensSpec
	:	#( TOKENS ( tokenSpec )+ )
	;

tokenSpec
	:
	TOKEN_REF
	( OPEN_ELEMENT_OPTION ID )?
	|
		#(
			ASSIGN
			TOKEN_REF
			( OPEN_ELEMENT_OPTION ID )?
			KEYWORD
		)
	;

rules
	:
	(
		rule
		|
			constructorRule
	)+
	;

rule
	:
	#(	RULE id:ID
		(modifier)?
		{out(#id.getText());}
		(optionsSpec)?
		( ruleAttributeDecls )?
		( action )?
		{out(" : ");}
		b:block[false]
		(exceptionGroup)?
		EOR {out(";\n");}
         )
    ;


constructorRule
	:
	#(	ATTRIBUTE_COLON
		RULE_REF
		( attributeItem )+
	)
	;


modifier
{out(#modifier.getText()); out(" ");}
	:	"protected"
	|	"public"
	|	"private"
	|	"fragment"
	;


block[boolean forceParens]
{
int numAlts = countAltsForBlock(#block);
}
    :   #(  BLOCK {if ( forceParens||numAlts>1 ) out(" (");}
            (optionsSpec {out(" : ");} )?
            alternative ( {out(" | ");} alternative )*
            EOB   {if ( forceParens||numAlts>1 ) out(")");}
         )
    ;

countAltsForBlock returns [int n=0]
    :   #( BLOCK (OPTIONS)? (ALT {n++;})+ EOB )
	;

alternative
    :   #( ALT (element)+ EOA )
    ;

exceptionGroup
	:	( exceptionSpec )+
    ;

exceptionSpec
    :   #("exception" ( arg_decl )? ( exceptionHandler )*)
    ;

arg_decl
	:
	#(	RBRACKET
		(	STRING_LITERAL		// anonymous StringTemplate
			|
				ID		// StringTemplate name
		)
		( 	COLON
			( assignment )+
		)?
	)
	;
	

exceptionHandler
    :    #("catch" arg_decl action)
    ;


element
	:
	atom
	( ast_suffix )?
	|
		#(NOT {out("~");} atom) 
	|
		#(RANGE atom {out("..");} atom)
	|	#(CHAR_RANGE atom {out("..");} atom) ( ast_suffix )?
	|
		l:attrLabel		{out(#l.getText()+"=");}
		(	atom
			|
				#( NOT atom )
			|
				baseebnf
			|
				treeMatcher
			|
				attributeElement
		)
    |	attributeElement // instantiate
    |   ebnf
    |   treeMatcher ( BANG )?
    |   #( SYNPRED block[true] ) {out("=>");}
    |   action
    |   sempred
    |   spred:SYN_SEMPRED
    	{
    	  String name = spred.getText();
    	  GrammarAST predAST=grammar.getSyntacticPredicate(name);
    	  block(predAST, true);
    	  out("=>");
    	}
    |   EPSILON
    ;


// Templated actions
action
	:
	#(	RCURLY
		template
	)
	;


sempred
	:
	#(	SEMPRED
		template
	)
	;


template
	:
	(	STRING_LITERAL		// anonymous StringTemplate
		|
			ID		// StringTemplate name
	)
	(
		( assignment )+
	)?
	;


assignment
	:
	#(	ID
		(	attribute
			|
				TOKEN_REF	// symbolic constants
			|
				STRING_LITERAL
		)
	)
	;


attrLabel
	:
	(	// assign but do not remove AST
		a:LABEL_ATTRIBUTE
	|	// assign and remove AST
		ASSIGN_ATTRIBUTE
	)
	( arrayQualifier )?
	(
		idQualifier
	)*
	;


elementPrint
	:
	atom
	|
		#(NOT {out("~");} atom) 
	|
		#(RANGE atom {out("..");} atom)
    |   #(CHAR_RANGE atom {out("..");} atom) ( ast_suffix )?
    |	(	aa:ASSIGN_ATTRIBUTE		{out(#aa.getText()+"=");}
		|
			l:LABEL_ATTRIBUTE	{out(#l.getText()+"=");}
	)
	( arrayQualifier )?
	( idQualifier )*
	atom 
    |   ebnf
    |   treeMatcher
    |   #( SYNPRED block[true] ) {out("=>");}
    |   action
    |   sempred
    |   spred:SYN_SEMPRED
    	{
    	  String name = spred.getText();
    	  GrammarAST predAST=grammar.getSyntacticPredicate(name);
    	  block(predAST, true);
    	  out("=>");
    	}
    |   EPSILON
    ;


ebnf
	:
	#( BANGEDUP baseebnf )
	|
		baseebnf
	;


baseebnf:   block[true] {out(" ");}
    |   #( OPTIONAL block[true] ) {out("? ");}
    |   #( CLOSURE block[true] )  {out("* ");}
    |   #( POSITIVE_CLOSURE block[true] ) {out("+ ");}
    ;

treeMatcher
	:
	#(TREE_BEGIN {out(" ^(");} element (element)* {out(") ");} )
	;


atom
{out(" ");}
    :   (	#( RULE_REF		{out(#atom.toString());}
			   (rarg:ARG_ACTION	{out("["+#rarg.toString()+"]");})?
			   (ast_suffix)?
             )
		|   #( TOKEN_REF		{out(#atom.toString());} 
			   (targ:ARG_ACTION	{out("["+#targ.toString()+"]");} )?
			   (ast_suffix)?
             )
		|   #( KEYWORD	{out(#atom.toString());}
			   (ast_suffix)?
             )
		|   #( STRING_LITERAL	{out(#atom.toString());}
			   (ast_suffix)?
             )
		|   #( WILDCARD		{out(#atom.toString());}
			   (ast_suffix)?
             )
		)
		{out(" ");}
    |   set
    ;


attribute
	:
	ATTRIBUTE
	( arrayQualifier )?
	( idQualifier )*
	;


ast_suffix
	:	CARET {out("^");}
	|	BANG  {out("!");}
	;

set
    :   #( SET
    	   {out("(");} setElement ({out("|");} setElement)* {out(")");}
           (ast_suffix)?
    	 )
    ;

setElement
    :   (	KEYWORD    {out(#setElement.toString());}
		|   TOKEN_REF		{out(#setElement.toString());}
		|   STRING_LITERAL	{out(#setElement.toString());}
		)
    |	#(CHAR_RANGE c1:KEYWORD c2:KEYWORD)
    	{out(#c1.getText()+
    	     ".."+
    	     #c2.getText());
    	}
    ;

// ATTRIBUTE syntax
attributeGroup
	:
	#( 
		ATTRIBUTE_GROUP
		(
			attributeItem
			|
				STRING_LITERAL
		)*
	)
	;

attributeBlock
	:
	(
		attributeItem
	)*
	|
		sempred
		(
			attributeItem
		)*
		( 
			OR
			attributeBlock
		)
	;


attributeStatements
	:
	#(	ATTRIBUTE_STMT
		attributeBlock
	)
	;



attributeItem
	:
	( attrLabel )?
	attributeElement
	;

attributeElement
	:
	attribute
	(
		CARET
	)?
	|
		attributeGroup
		( CARET )?
	|
		attributeStatements
	|
		CURRENT_TREE
		( BANG )?
	|
		LPAREN
		(
			attributeItem
		)*
		RPAREN
	|
		treeConstructor
	;


treeConstructor
	:
	#(
		TREE_CONSTRUCTOR
		attributeItem
		(
			attributeItem
		)+
	)
	;
// END ATTRIBUTE syntax

