/*
 [The "BSD licence"]
 Copyright (c) 2006 Loring Craymer
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.runtime;

public class DefaultPayload implements Payload {
	int type = -1;
	String text = null;
	
	public DefaultPayload() {
	}
	
	public DefaultPayload(int type_, String txt) {
		text = txt;
		type = type_;
	}
	
	public DefaultPayload(DefaultPayload pay) {
		text = pay.text;
		type = pay.type;
	}
	
	/* (non-Javadoc)
	 * @see antlr.Payload#getText()
	 */
	public String getText() {
		return text;
	}

	/* (non-Javadoc)
	 * @see antlr.Payload#getType()
	 */
	public int getType() {
		return type;
	}
	
	/* (non-Javadoc)
	 * @see antlr.Payload#setText(java.lang.String)
	 */
	public void setText(String text_) {
		text = text_;
	}
	
	/* (non-Javadoc)
	 * @see antlr.Payload#setType(int)
	 */
	public void setType(int type_) {
		type = type_;
	}
	
	public boolean equals(Object o) {
		DefaultPayload p = (DefaultPayload) o;
		
		if (type != p.type)
			return false;
		
		if (text != p.text)
			return false;
		
		return true;
	}
	
	public Carrier wrap() {
		return new DefaultCarrier(this);
	}
	
	public String toString() {
		return text;
	}
}
