/*
 [The "BSD licence"]
 Copyright (c) 2006 Loring Craymer
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.antlr_yggdrasil.runtime;


public class PayloadToken implements Token, Payload {
	String text = null;
    protected int type;
	protected int line;
	protected int charPositionInLine = -1; // set to invalid position
	protected int channel=DEFAULT_CHANNEL;
	protected CharSequence input;

	/** What token number is this from 0..n-1 tokens */
	protected int index;

    /** The char position into the input buffer where this token starts */
	protected int start;
	
	/** The char position into the input buffer where this token stops */
	protected int stop;
	
	public PayloadToken() {
		
	}


	public PayloadToken(int type_) {
		type = type_;
	}
	
	public PayloadToken(PayloadToken pay) {
		text = pay.text;
		type = pay.type;
		line = pay.line;
		charPositionInLine = pay.charPositionInLine;
		channel = pay.channel;
		input = pay.input;
	}
	
	public PayloadToken(int type_, String txt) {
		type = type_;
		text = txt;
	}

	public int getLine() {
		return line;
	}

	public void setLine(int line) {
		this.line = line;
	}

	public int getCharPositionInLine() {
		return charPositionInLine;
	}

	public void setCharPositionInLine(int pos) {
		this.charPositionInLine = pos;
	}

	public int getChannel() {
		return channel;
	}

	public void setChannel(int channel) {
		this.channel = channel;
	}

	public int getTokenIndex() {
		return index;
	}

	public void setTokenIndex(int index) {
		this.index = index;
	}
	
	public void setStartIndex(int idx) {
		start = idx;
	}
	
	public int getStartIndex() {
		return start;
	}
	
	public void setStopIndex(int idx) {
		stop = idx;
	}
	
	public int getStopIndex() {
		return stop;
	}
	
	public void setInput(CharSequence in) {
		input = in;
	}
	
	public CharSequence getInput() {
		return input;
	}

	public String getText() {
		if ( text!=null ) {
			return text;
		}
		if ( input==null ) {
			return null;
		}
		
        text = input.subSequence(start,stop).toString();
        return text;
	}

	public int getType() {
		return type;
	}

	public void setText(String text_) {
		text = text_;
	}

	public void setType(int type_) {
		type = type_;
	}
	
	public String toString() {
		return getText();
	}

	public Carrier wrap() {
		return new DefaultCarrier(this);
	}

}
