/*
 [The "BSD licence"]
 Copyright (c) 2006 Loring Craymer
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Test grammar for automated tree grammar production

header {
}



lexer grammar TestLexer;
options {
	k=2;
	userTemplates = "user";
	outputVocab = "TEST";
	factory = "TestParserFactory";
}


WS	:	(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			' '
		|	'\t'
		|	'\r' '\n'
		|	'\r'
		|	'\n'
		)
		{ skip }
	;

COMMENT : 
	( SL_COMMENT | ML_COMMENT )
	{ skip }
	;

fragment
SL_COMMENT :
	"//"
	( ~('\n'|'\r') )*
	(
		/*	'\r' '\n' can be matched in one alternative or by matching
			'\r' and then in the next token.  The language
			that allows both "\r\n" and "\r" and "\n" to all be valid
			newline is ambiguous.  Consequently, the resulting grammar
			must be ambiguous.  I'm shutting this warning off.
		 */
			'\r' '\n'
		|	'\r'
		|	'\n'
	)
	;


fragment
ML_COMMENT :
	"/*"!
	(
		/*	'\r' '\n' can be matched in one alternative or by matching
			'\r' and then in the next token.  The language
			that allows both "\r\n" and "\r" and "\n" to all be valid
			newline is ambiguous.  Consequently, the resulting grammar
			must be ambiguous.  I'm shutting this warning off.
		 */
		options {
			greedy=false;  // make it exit upon "*/"
		}
	:	'\r' '\n'
	|	'\r'
	|	'\n'
	|	~('\n'|'\r')
	)*
	"*/"!
	;

A : 'A' ;
B : 'B' ;
C : 'C' ;
D : 'D' ;
E : 'E' ;
F : 'F' ;
G : 'G' ;
H : 'H' ;
I : 'I' ;
J : 'J' ;
K : 'K' ;
L : 'L' ;
M : 'M' ;
N : 'N' ;
O : 'O' ;
P : 'P' ;
Q : 'Q' ;
R : 'R' ;
S : 'S' ;
T : 'T' ;
U : 'U' ;
V : 'V' ;
W : 'W' ;
X : 'X' ;
Y : 'Y' ;
Z : 'Z' ;
Aa : "Aa" ;
Ba : "Ba" ;
Ca : "Ca" ;
Da : "Da" ;
Ea : "Ea" ;
Fa : "Fa" ;
Ga : "Ga" ;
Ha : "Ha" ;
Ia : "Ia" ;
Ja : "Ja" ;
Ka : "Ka" ;
La : "La" ;
Ma : "Ma" ;
Na : "Na" ;
Pa : "Pa" ;
Qa : "Qa" ;
Ra : "Ra" ;
Sa : "Sa" ;
Ta : "Ta" ;
Ua : "Ua" ;
Wa : "Wa" ;
Xa : "Xa" ;
Ya : "Ya" ;
Za : "Za" ;
Ab : "Ab" ;
Bb : "Bb" ;
Cb : "Cb" ;
Db : "Db" ;
Eb : "Eb" ;
Fb : "Fb" ;
Gb : "Gb" ;
Hb : "Hb" ;
Ib : "Ib" ;
Jb : "Jb" ;
Kb : "Kb" ;
Lb : "Lb" ;
Mb : "Mb" ;
Nb : "Nb" ;
Ob : "Ob" ;
Pb : "Pb" ;
Qb : "Qb" ;
Rb : "Rb" ;
Sb : "Sb" ;
Tb : "Tb" ;
Ub : "Ub" ;
Wb : "Wb" ;
Xb : "Xb" ;
Yb : "Yb" ;
Zb : "Zb" ;
Ac : "Ac" ;
Bc : "Bc" ;
Cc : "Cc" ;
Dc : "Dc" ;
Ec : "Ec" ;
Fc : "Fc" ;
Gc : "Gc" ;
Hc : "Hc" ;
Ic : "Ic" ;
Jc : "Jc" ;
Kc : "Kc" ;
Lc : "Lc" ;
Mc : "Mc" ;
Nc : "Nc" ;
Oc : "Oc" ;
Pc : "Pc" ;
Qc : "Qc" ;
Rc : "Rc" ;
Sc : "Sc" ;
Tc : "Tc" ;
Uc : "Uc" ;
Wc : "Wc" ;
Xc : "Xc" ;
Yc : "Yc" ;
Zc : "Zc" ;
